from rox import g
import top
import gobject
from cell_bar import CellRendererBar
from support import pretty_size

MOUNT = 0
RESERVED = 1
USED = 2
FREE = 3
SIZE = 4
USED_TXT = 5

class FileSystems(g.ScrolledWindow):
	def __init__(self, draw_to_scale):
		g.ScrolledWindow.__init__(self)

		self.set_shadow_type(g.SHADOW_IN)
		self.set_policy(g.POLICY_NEVER, g.POLICY_NEVER)
		self.unset_flags(g.CAN_FOCUS)

		self.model = list = g.ListStore(str, float, float, float,
						str, str)

		view = g.TreeView(list)
		self.add(view)
		sel = view.get_selection()
		sel.set_mode(g.SELECTION_NONE)

		view.insert_column_with_attributes(0, _('Mount point'),
				g.CellRendererText(), text = MOUNT)

		view.insert_column_with_attributes(1, _('Size'),
				g.CellRendererText(), text = SIZE)

		view.insert_column_with_attributes(2, _('Used'),
				g.CellRendererText(), text = USED_TXT)

		self.cell_bar = CellRendererBar()
		self.cell_bar.scaled = draw_to_scale
		view.insert_column_with_attributes(3,
				_('Space usage (reserved, used, free)'),
				self.cell_bar,
				part1 = RESERVED, part2 = USED, part3 = FREE)

		self.update()

		self.timeout = gobject.timeout_add(2000, self.update)
	
	def toggle_scaled(self):
		self.cell_bar.scaled = not self.cell_bar.scaled
		self.update()
	
	def update(self):
		mounts = top.get_mountlist()
		mounts.sort()

		model = self.model
		iter = model.get_iter_first()
		
		total = 0

		for m in mounts:
			usage = top.get_fsusage(m)
			if not usage: continue
			free, reserved, used_user, blocks, block_size = usage

			while 1:
				if not iter:
					new = model.append()
					break
				diff = cmp(model[iter][MOUNT], m)
				if diff == 0:
					# Same
					new = iter
					break
				elif diff > 0:
					new = model.insert_before(iter)
					break

				to_delete = iter
				iter = model.iter_next(iter)
				model.remove(to_delete)
	
			model.set(new,
				MOUNT, m,
				RESERVED, reserved,
				USED, used_user,
				FREE, free,
				SIZE, pretty_size(blocks * block_size),
				USED_TXT, "%.2f%%" %
					(100 * used_user / (used_user + free)))

			iter = model.iter_next(new)
			
			total = max(total, free + used_user + reserved)
		
		while iter:
			to_delete = iter
			iter = model.iter_next(iter)
			model.remove(to_delete)
		
		self.cell_bar.max = max(total, 1)
		
		return True
